package com.yihu.hos.arbiter.services;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.hos.arbiter.common.constants.Constants;
import com.yihu.hos.arbiter.models.BrokerServer;
import com.yihu.hos.web.framework.model.bo.ServiceFlow;
import com.yihu.hos.core.http.HTTPResponse;
import com.yihu.hos.core.http.HttpClientKit;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * Broker原则上具有等同性，这样Arbiter无论选择了哪个Broker能提供的服务都是一样的。
 * 但是因为Broker上还是会运行一些定时的采集任务，这些采集任务如果是多台Broker运行的话，可能会引起数据重复问题。
 * 所以在事件触发时需要做一些策略的调整：
 * 1.实时任务，通知所有的Broker进行更新路由
 * 2.采集任务，只通知其中的一台进行更新路由
 *
 * @created Airhead 2016/8/16.
 */
@Service("serviceFlowService")
public class ServiceFlowService {
    private static final Logger logger = LogManager.getLogger(BrokerServerService.class);
    @Autowired
    private MongoOperations mongoOperations;
    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private BrokerServerService brokerServerService;


    public void save(ServiceFlow serviceFlow) {
        Query query = new Query();
        query.addCriteria(Criteria.where("routeCode").is(serviceFlow.getRouteCode()));

        Update update = new Update();
        update.set("routeCode", serviceFlow.getRouteCode());
        update.set("handleFiles", serviceFlow.getHandleFiles());
        update.set("updated", serviceFlow.getUpdated());
        update.set("flowType", serviceFlow.getFlowType());

        mongoOperations.upsert(query, update, ServiceFlow.class);
    }

    public String get(String serviceName) {
        return null;
    }

    public List<ServiceFlow> getAll() {
        return mongoOperations.findAll(ServiceFlow.class);
    }

    public String put(String serviceName, String ClientInfo) {
        return null;
    }

    public void flowRefresh(String msg) {
        flowController("post", "/esb/serviceFlow", msg);
    }

    public void processorAdded(String msg) {
        flowController("post", "/esb/processor", msg);
    }

    public void processorDataChanged(String msg) {
        flowController("put", "/esb/processor", msg);
    }

    public void processorDataDeleted(String msg) {
        flowController("delete", "/esb/processor", msg);
    }

    public void routeDefineAdded(String msg) {
        flowController("post", "/esb/route", msg);
    }

    public void routeDefineChanged(String msg) {
        flowController("put", "/esb/route", msg);
    }

    public void routeDefineDeleted(String msg) {
        flowController("delete", "/esb/route", msg);
    }

    public void routeClassAdded(String msg) {
        flowController("post", "/esb/genRoute", msg);
    }

    public void routeClassChanged(String msg) {
        flowController("put", "/esb/updateRoute", msg);
    }

    public void processorClassAdded(String msg) {
        flowController("post", "/esb/genProcessor", msg);
    }

    public void processorClassChanged(String msg) {
        flowController("put", "/esb/genProcessor", msg);
    }

    public void brokerServerOn(String msg) {
        List<ServiceFlow> serviceFlowList = getAll();
        serviceFlowList.forEach(serviceFlow -> {
            try {
                String serviceFlowMsg = objectMapper.writeValueAsString(serviceFlow);
                flowController("post", "/esb/serviceFlow", serviceFlowMsg);
            } catch (JsonProcessingException e) {
                e.printStackTrace();
            }
        });

    }


    /**
     * SAAS化的管理端过来的消息会被proxy进行中转，之后发送到终端的Arbiter对Broker进行实际的控制。
     *
     * @param header 消息头部信息
     * @param msg    消息
     */
    public void proxy(Map<String, Object> header, String msg) {
    }

    private void flowController(String method, String path, String msg) {
        try {
            ServiceFlow serviceFlow = objectMapper.readValue(msg, ServiceFlow.class);
            this.save(serviceFlow);

            boolean one = Constants.JAVA.equals(serviceFlow.getFlowType());   //有cron表达式，就是采集任务。
            if (one) {
                List<BrokerServer> flowOnBroker = brokerServerService.getFlowOnBroker(serviceFlow.getRouteCode());
                if (flowOnBroker != null && flowOnBroker.size() != 0) {
                    return;
                }
            }

            List<BrokerServer> brokerServerList = brokerServerService.get(one);
            for (BrokerServer brokerServer : brokerServerList) {
                if (brokerServer.isFlowOn(serviceFlow.getRouteCode())) {
                    continue;
                }
                switch (method) {
                    case "post":
                        HTTPResponse response = HttpClientKit.post(brokerServer.getURL() + path, msg);
                        if (response.getStatusCode() == 200){
                            String body = response.getBody();
                        }
                        break;
                    case "put":
                        HttpClientKit.put(brokerServer.getURL() + path, msg);
                        break;
                    case "delete":
                        HttpClientKit.delete(brokerServer.getURL() + path, msg);
                        break;
                    default:
                        break;
                }
            }

        } catch (IOException e) {
            e.printStackTrace();
            logger.error(e.getMessage());
        }
    }


}
